#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Copyright 2016 Fedele Mantuano (https://twitter.com/fedelemantuano)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import argparse
import os
import runpy

try:
    from collections import ChainMap
except ImportError:
    from chainmap import ChainMap

from tikapp import TikaApp

current = os.path.realpath(os.path.dirname(__file__))

__version__ = runpy.run_path(
    os.path.join(current, "version.py"))["__version__"]


def get_args():
    parser = argparse.ArgumentParser(
        description="Wrapper for Apache Tika App.",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parsing_group = parser.add_mutually_exclusive_group(required=True)
    parsing_group.add_argument(
        "-f",
        "--file",
        dest="file",
        help="File to submit")
    parsing_group.add_argument(
        "-p",
        "--payload",
        dest="payload",
        help="Base64 payload to submit")

    parser.add_argument(
        "-j",
        "--jar",
        dest="jar",
        help="Apache Tika app JAR")

    parser.add_argument(
        "-d",
        "--detect",
        dest="detect",
        action="store_true",
        help="Detect document type")

    parser.add_argument(
        "-t",
        "--text",
        dest="text",
        action="store_true",
        help="Output plain text content")

    parser.add_argument(
        "-l",
        "--language",
        dest="language",
        action="store_true",
        help="Output only language")

    parser.add_argument(
        "-a",
        "--all",
        dest="all",
        action="store_true",
        help="Output metadata and content from all embedded files")

    parser.add_argument(
        '-v',
        '--version',
        action='version',
        version='%(prog)s {}'.format(__version__))

    return parser.parse_args()


def main():
    args = get_args()

    command_line = dict()
    if args.jar:
        command_line = {"TIKA_APP_JAR": args.jar}

    defaults = {"TIKA_APP_JAR": "/opt/tika/tika-app-1.15.jar"}
    options = ChainMap(command_line, os.environ, defaults)

    tika = TikaApp(options['TIKA_APP_JAR'])

    try:
        if args.file:
            f = args.file

            if args.detect:
                print(tika.detect_content_type(path=f))

            if args.text:
                print(tika.extract_only_content(path=f))

            if args.language:
                print(tika.detect_language(path=f))

            if args.all:
                print(tika.extract_all_content(path=f, pretty_print=True))

        elif args.payload:
            p = args.payload

            if args.detect:
                print(tika.detect_content_type(payload=p))

            if args.text:
                print(tika.extract_only_content(payload=p))

            if args.language:
                print(tika.detect_language(payload=p))

            if args.all:
                print(tika.extract_all_content(payload=p, pretty_print=True))

    except IOError:
        pass


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Copyright 2016 Fedele Mantuano (https://twitter.com/fedelemantuano)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from __future__ import unicode_literals
import logging
import os
import subprocess

import six
from .exceptions import TikaAppJarError
from .utils import file_path, clean, sanitize

try:
    import simplejson as json
except ImportError:
    import json


log = logging.getLogger(__name__)


class TikaApp(object):

    def __init__(self, file_jar=None, memory_allocation=None):
        self.file_jar = file_jar
        self.memory_allocation = memory_allocation

    def __repr__(self):
        class_name = type(self).__name__
        return "{}({!r}, {!r})".format(
            class_name, self.file_jar, self.memory_allocation)

    @property
    def file_jar(self):
        return self._file_jar

    @file_jar.setter
    def file_jar(self, value):
        if not value or not os.path.exists(value):
            msg = "Tika app jar not valid"
            log.exception(msg)
            raise TikaAppJarError(msg)

        self._file_jar = value

    @property
    def memory_allocation(self):
        return self._memory_allocation

    @memory_allocation.setter
    def memory_allocation(self, value):
        self._memory_allocation = value

    @property
    def help(self):
        return self._command_template(["--help"])

    @sanitize
    def _command_template(self, switches):
        """Template for Tika app commands

        Args:
            switches (list): list of switches to Tika app Jar

        Return:
            Standard output data (unicode Python 2, str Python 3)
        """

        command = ["java", "-jar", self.file_jar, "-eUTF-8"]

        if self.memory_allocation:
            command.append("-Xmx{}".format(self.memory_allocation))

        command.extend(switches)

        if six.PY2:
            with open(os.devnull, "w") as devnull:
                out = subprocess.Popen(
                    command, stdin=subprocess.PIPE,
                    stdout=subprocess.PIPE, stderr=devnull)
        elif six.PY3:
            out = subprocess.Popen(
                command, stdin=subprocess.PIPE,
                stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)

        stdoutdata, _ = out.communicate()

        return stdoutdata.decode("utf-8").strip()

    def generic(self, switches=["--help"]):
        """Generic method. Default display help"""
        return self._command_template(switches)

    @clean
    def detect_content_type(self, path=None, payload=None):
        """Return the content type of passed file or payload.

        Args:
            path (string): Path of file to analyze
            payload (string): Payload base64 to analyze
        """

        f = file_path(path, payload)
        switches = ["-d", f]
        result = self._command_template(switches).lower()
        return result, path, f

    @clean
    def extract_only_content(self, path=None, payload=None):
        """Return only the text content of passed file

        Args:
            path (string): Path of file to analyze
            payload (string): Payload base64 to analyze
        """

        f = file_path(path, payload)
        switches = ["-t", f]
        result = self._command_template(switches)
        return result, path, f

    @clean
    def detect_language(self, path=None, payload=None):
        """Return the language of passed file or payload.

        Args:
            path (string): Path of file to analyze
            payload (string): Payload base64 to analyze
        """

        f = file_path(path, payload)
        switches = ["-l", f]
        result = self._command_template(switches)
        return result, path, f

    @clean
    def extract_all_content(self, path=None, payload=None,
                            pretty_print=False, convert_to_obj=False):
        """Return a JSON of all contents and metadata of passed file

        Args:
            path (string): Path of file to analyze
            payload (string): Payload base64 to analyze
            pretty_print (boolean): If True adds newlines and whitespace,
                                    for better readability
            convert_to_obj (boolean): If True convert JSON in object
        """
        f = file_path(path, payload)

        if pretty_print:
            switches = ["-J", "-t", "-r", f]
        else:
            switches = ["-J", "-t", f]

        result = self._command_template(switches)

        if result and convert_to_obj:
            result = json.loads(result, encoding="utf-8")

        return result, path, f



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Copyright 2016 Fedele Mantuano (https://twitter.com/fedelemantuano)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import os
import runpy
from setuptools import setup


current = os.path.realpath(os.path.dirname(__file__))

with open(os.path.join(current, 'README')) as f:
    long_description = f.read().strip()

with open(os.path.join(current, 'requirements.txt')) as f:
    requires = f.read().splitlines()

__version__ = runpy.run_path(
    os.path.join(current, "tikapp", "version.py"))["__version__"]


setup(
    name="tika-app",
    description="Python client for Apache Tika App",
    license="Apache License, Version 2.0",
    url="https://github.com/fedelemantuano/tika-app-python",
    long_description=long_description,
    version=__version__,
    author="Fedele Mantuano",
    author_email="mantuano.fedele@gmail.com",
    maintainer="Fedele Mantuano",
    maintainer_email="mantuano.fedele@gmail.com",
    packages=["tikapp"],
    platforms=["Linux"],
    keywords=["tika", "apache", "toolkit"],
    classifiers=[
        "License :: OSI Approved :: Apache Software License",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
    ],
    install_requires=requires,
    entry_points={"console_scripts": [
        "tikapp = tikapp.__main__:main"]},
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Copyright 2016 Fedele Mantuano (https://twitter.com/fedelemantuano)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from __future__ import unicode_literals
import base64
import logging
import os
import tempfile
from unicodedata import normalize
from .exceptions import FilePathError


log = logging.getLogger(__name__)


def sanitize(func):
    """ NFC is the normalization form recommended by W3C. """

    def wrapper(*args, **kwargs):
        return normalize('NFC', func(*args, **kwargs))
    return wrapper


def clean(func):
    def wrapper(*args, **kwargs):
        out, given_path, path = func(*args, **kwargs)

        try:
            if not given_path:
                os.remove(path)
        except OSError:
            pass

        return out

    return wrapper


def file_path(path=None, payload=None):
    """Given a file path or payload return a file path

    Args:
        path (string): path of real file
        payload(string): payload in base64 of file

    Return:
        Path of file
    """
    f = path if path else write_payload(payload)

    if not os.path.exists(f):
        msg = "File {!r} does not exist".format(f)
        log.exception(msg)
        raise FilePathError(msg)

    return f


def write_payload(payload):
    """Write a base64 payload on temp file

    Args:
        payload (string): payload in base64

    Return:
        Path of file
    """

    temp = tempfile.mkstemp()[1]

    with open(temp, 'wb') as f:
        payload = base64.b64decode(payload)
        f.write(payload)

    return temp
